/*****************************************
 * Large Assignment 1: L-Systems
 * Name:    Daniel Morales-Garcia
 * E-mail:  dmorales33@unm.edu
 * Course:      CS491, Computational Fabrication
 * Submitted:    9/14/2023
 *
 * This program includes several different features.
* Create L-System for a bush.
* Create L-System for a Dumb Cane plant
* Create L-System for a tree
***********************************************/

import java.util.Random;

class DanielMoralesLA1Class{
  
  Random rand = new Random();
  
  // axiom is 'X' for bushRules and treeRules, and 'C' for dumbCaneRules 
  String axiom = "X";

  String [][] bushRules = {{"F", "F", "F"},
                       {"X", "F[-X][+X]FX", "F[-X][+X]FX"},
                       {"+", "+", "+"},
                       {"-", "-", "-"},
                       {"[", "[", "["},
                       {"]", "]", "]"}};
                       
                       
  String [][] treeRules = { {"F", "F", "FF", "FF", "F", "F"},
                      {"X", "F[+X][-X]", "F[-X]F[+X]F[-X]", "XF[+X]", "XF[-X]", "F[+X]F[-X]F[+X]"},
                      {"+", "+", "+", "+", "+", "+"},
                      {"-", "-", "-", "-", "-", "-"},
                      {"[", "[", "[", "[", "[", "["},
                      {"]", "]", "]", "]", "]", "]"}};
                       
  String [][] dumbCaneRules = {{"F", "F[B]F[B]", "F[B]F[B]"},
                      {"B", "B", "B"},
                       {"X", "X", "X"},
                       {"+", "+", "+"},
                       {"-", "-", "-"},
                       {"[", "[", "["},
                       {"]", "]", "]"}};
                       



                      
                     
                       
  String computedWord;
  int iterations;
  float size, angle, scaleFactor;
  
  DanielMoralesLA1Class(float size, float angle, float scaleFactor){
    this.size = size;
    this.angle = angle;
    this.scaleFactor = scaleFactor;
    // Computed Word start as 'X' for bushRules and treeRules, and 'C' for dumbCaneRules 
    computedWord = axiom;
    iterations = 0;
  }
  
  void iteration(){
    
    // change to any of the rules[][] above
    String[][] rules = treeRules;
    char s;
    String newWord = "";
    size = size * scaleFactor;
    
    for (int i = 0; i < computedWord.length(); i++){
      s = computedWord.charAt(i);
      for(int j = 0; j < rules.length; j++){
        if (s == rules[j][0].charAt(0)){
          int randomIndex = 1 + rand.nextInt(rules[j].length - 1);
          newWord = newWord + rules[j][randomIndex];
          break;
        }
      }
    }
    iterations++;
    computedWord = newWord;
    println(iterations + ": " + computedWord);
  }
  
  void drawLSystem(Turtle t){
    char s;
    int leafStemLength;
    int leftOrRight;

    for (int i = 0; i < computedWord.length(); i++){
      s= computedWord.charAt(i);
      
      if (s == 'C'){
        t.right(.3);
        t.forward(size/10);
        for(int k = 1; k < 9; k++){
          t.push();
          t.right(45 * k);
          t.forward(size/10);
          t.pop();
          
        }
      }     
      else if (s == 'B'){
        
        // determining if leaf is going left or right
        leftOrRight = rand.nextInt(2);
        if (leftOrRight == 0){
          t.left(angle);
        }
        else{
          t.right(angle);
        }
        
        leafStemLength = rand.nextInt(7);
        // making a random drooping affect for leaf
        for (int j = 0; j < leafStemLength; j++){
          t.forward(size/10);
          if (leftOrRight == 0){
            t.left(10);
          }
          else{
            t.right(10);
          }
        }
        leaf(t, leftOrRight, leafStemLength);
      }
      else if (s == 'F'){
        //t.right(random(-10, 10));
        t.forward(size);
        //drawSemicircle(t, 100, 5);
      }
      else if (s == '+')
        t.right(angle);
      else if (s == '-')
        t.left(angle);
      else if (s == '[')
        t.push();
      else if (s == ']')
        t.pop();
      else if (s == 'X'){
         leaf(t);
      }
    }
  }
 
// Leaf for Dumb Cane
void leaf(Turtle t, int leftOrRight, int leafStemLength) {
  // Save the current drawing state
  pushMatrix();

  // Move to the turtle's position
  translate(t.getX(), t.getY());

  if (leftOrRight == 1){
    if (leafStemLength < 4)
      rotate(random(radians(90), radians(120)));
    else
      rotate(random(radians(120), radians(180)));
  }
 else if(leftOrRight == 0){
    if (leafStemLength < 2)
      rotate(random(radians(270), radians(240)));
    else
      rotate(random(radians(180), radians(240)));
  }

  // Define an array of shades of green
  color[] greens = {
  color(34, 139, 34), 
  color(0, 128, 0),   
  color(60, 179, 113) 
};

  // Randomly select one of the four shades of green
  fill(greens[int(random(3))]);
  
  noStroke();

  beginShape();
  vertex(0, 0);   
  bezierVertex(30, -15, 20, -45, 0, -60); // Right side
  bezierVertex(-30, -15, -15, -15, 0, 0); // Left side
  endShape();
  
  stroke(83, 53, 10); // Makes stroke brown

  // Restore the previous drawing state
  popMatrix();
}



// Leaf for bush and tree
void leaf(Turtle t) {
  // Save the current drawing state
  pushMatrix();

  // Move to the turtle's position
  translate(t.getX(), t.getY());

  // Rotate to a random direction
  float randomAngle = random(TWO_PI); // Select a random angle between 0 and 2*PI (which covers a full circle)
  rotate(randomAngle);

  // Define an array of shades of green
  color[] greens = {
    color(34, 139, 34),
    color(0, 128, 0),
    color(60, 179, 113)
  };

  // Randomly select one of the four shades of green
  fill(greens[int(random(3))]);
  
  noStroke();

  beginShape();
  vertex(0, 0);
  bezierVertex(5, -2.5, 2.5, -7.5, 0, -10); // right side
  bezierVertex(-5, -7.5, -2.5, -2.5, 0, 0); // Left side
  endShape();
  
  stroke(83, 53, 10); // Makes stroke brown

  // Restore the previous drawing state
  popMatrix();
}

    
}
